using HermesSocketLibrary.db;
using HermesSocketLibrary.Requests.Messages;
using HermesSocketServer.Models;
using ILogger = Serilog.ILogger;

namespace HermesSocketServer.Requests
{
    public class UpdateTTSVoiceState : IRequest
    {
        public string Name => "update_tts_voice_state";
        public string[] RequiredKeys => ["voice", "state"];
        private Database _database;
        private ILogger _logger;

        public UpdateTTSVoiceState(Database database, ILogger logger)
        {
            _database = database;
            _logger = logger;
        }

        public Task<RequestResult> Grant(Channel channel, IDictionary<string, object> data)
        {
            var voiceId = data["voice"].ToString()!;
            var state = data["state"].ToString()?.ToLower() == "true";

            var voiceState = new TTSVoiceState()
            {
                Id = voiceId,
                UserId = channel.Id,
                Enabled = state,
            };

            var result = channel.VoiceStates.Set(voiceId, voiceState);
            if (result)
            {
                _logger.Information($"Updated voice state on channel [voice id: {voiceId}][state: {state}][channel: {channel.Id}]");
                return Task.FromResult(RequestResult.Successful(voiceState));
            }
            return Task.FromResult(RequestResult.Failed("Something went wrong when updating the database."));
        }
    }
}